package	jp.co.sds_corp.util.msg;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.msg.jms.SimpleJmsQueue;
import jp.co.sds_corp.util.msg.jms.SimpleJmsTopic;

/**
* {@link IMessageAdapter}を生成するFactoryクラスです。
Type要素、もしくはAdapter要素によって生成するクラスを選択します。
<pre>
&lt;Adapter class="jp.co.sds_corp.util.msg.jms.SimpleJmsTopic" /&gt;
&lt;Adapter type="jmsQueue" /&gt;
&lt;InitialContext&gt;weblogic.jndi.WLInitialContextFactory&lt;/InitialContext&gt;
&lt;ConnectionFactory&gt;jms/topic/ConnectionFactory&lt;/ConnectionFactory&gt;
&lt;URL&gt;http://localhost:8080&lt;/URL&gt;
&lt;User&gt;user&lt;/User&gt;
&lt;Password&gt;passwd&lt;/Password&gt;
&lt;Receive&gt;jms/topic/result&lt;/Receive&gt;
&lt;Send&gt;jms/topic/request&lt;/Send&gt;
</pre>
*/
public class MessageAdapterFactory implements Serializable
{
	public static final String ADAPTER_JMSQUEUE_KEY		= "jmsQueue";
	public static final String ADAPTER_JMSTOPIC_KEY		= "jmsTopic";
	public static final String ADAPTER_SOCKETTOPIC_KEY	= "socketTopic";
	public static final String ADAPTER_SOCKETQUEUE_KEY	= "socketQueue";

	public static final String ADAPTER_TYPE				= "type";
	public static final String ADAPTER_CLASS			= "class";

	public static final String INITIAL_CONTEXT			= "InitialContext";
	public static final String CONNECTION_NAME			= "Name";
	public static final String CONNECTION_FACTORY		= "ConnectionFactory";
	public static final String CONNECTION_URL			= "URL";
	public static final String CONNECTION_USER			= "User";
	public static final String CONNECTION_PASSWD		= "Password";
	public static final String DEST_SEND				= "Send";
	public static final String DEST_RECV				= "Receive";

	private Map<String,IMessageAdapter> adapters =
				new HashMap<String,IMessageAdapter>();

	/**
	* コンストラクタ
	*/
	public MessageAdapterFactory() {
		adapters.put( ADAPTER_JMSQUEUE_KEY,		new SimpleJmsQueue());
		adapters.put( ADAPTER_JMSTOPIC_KEY,		new SimpleJmsTopic());
/*
		adapters.put( ADAPTER_SOCKETTOPIC_KEY,	new SocketTopicPackage());
		adapters.put( ADAPTER_SOCKETQUEUE_KEY,	new SocketQueuePackage());
*/
	}

	/**
	* メッセージアダプタに接続します。
	* @param	setup	接続パラメータ
	* @return	接続したアダプタインスタンス
	*/
	public IMessageAdapter connect( Hashtable setup ) throws Exception {
		String type = (String )setup.get( ADAPTER_TYPE );
		String cls = (String )setup.get( ADAPTER_CLASS );
		IMessageAdapter adapter = lookup( type, cls );
		adapter.connect( setup );
		return adapter;
	}

	/**
	* メッセージアダプタに接続します。
	* @param	config	接続パラメータ
	* @return	接続したアダプタインスタンス
	*/
	public IMessageAdapter connect( IConfiguration config ) throws Exception {
		String type = config.getAttributes().get( ADAPTER_TYPE );
		String cls = config.getAttributes().get( ADAPTER_CLASS );
		IMessageAdapter adapter = lookup( type, cls );
		adapter.connect( config );
		return adapter;
	}

	/**
	* アダプターをルックアップします。
	* @param	type	アダプタキー
	* @return	アダプターインスタンス
	*/
	private IMessageAdapter lookup( String type, String cls ) throws Exception {
		IMessageAdapter adapter = (IMessageAdapter )adapters.get( type );
		if ( adapter == null ) {
			adapter = (IMessageAdapter )Class.forName( cls ).newInstance();
		}
		return adapter;
	}

}
