package	jp.co.sds_corp.util.msg.jms;

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.naming.Context;
import javax.naming.InitialContext;

import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfigSavable;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.msg.IExceptionListener;
import jp.co.sds_corp.util.msg.IMessage;
import jp.co.sds_corp.util.msg.IMessageAdapter;
import jp.co.sds_corp.util.msg.IReceiveListener;
import jp.co.sds_corp.util.msg.MessageAdapterFactory;
import jp.co.sds_corp.util.msg.message.jms.JmsMessage;

/**
* JMS メッセージングサービスの親クラス
*/
public abstract class AbstractJmsPackage
	implements IConfigSavable,
			ExceptionListener, MessageListener, IMessageAdapter,
			Serializable
{
	String jndiFactory;
	String jmsFactory;
	String url;
	String user;
	String passwd;
	String sendName;
	String recvName;

	protected Session		sndSession 		= null;
	protected Session		rcvSession 		= null;
	protected Connection	connection 		= null;
	protected MessageProducer	sender  	= null;
	protected MessageConsumer	receiver	= null;
	protected Destination	 recvDest		= null;
	protected Destination	 sendDest		= null;

	protected InitialContext jndi	   		= null;
	protected boolean		 wait	  		= false;
	protected int recv_timeout = 60000;

	protected LinkedList<IReceiveListener> listeners =
		new LinkedList<IReceiveListener>();
	protected LinkedList<IExceptionListener> exceptions =
		new LinkedList<IExceptionListener>();

	/**
	* ＪＭＳに接続します
	* @param	config	接続パラメータ
	*/
	public void connect( IConfiguration config ) throws Exception {
		jndiFactory = config.get( MessageAdapterFactory.INITIAL_CONTEXT );
		jmsFactory = config.get( MessageAdapterFactory.CONNECTION_FACTORY );
		url = config.get( MessageAdapterFactory.CONNECTION_URL );
		user = config.get( MessageAdapterFactory.CONNECTION_USER, null );
		passwd = config.get( MessageAdapterFactory.CONNECTION_PASSWD, null );
		sendName = config.get( MessageAdapterFactory.DEST_SEND, null );
		recvName = config.get( MessageAdapterFactory.DEST_RECV, null );

		connect( jndiFactory, jmsFactory, url, user, passwd );
	}

	/**
	* ＪＭＳに接続します
	* @param	env		接続パラメータ
	*/
	public void connect( Hashtable env ) throws Exception {
		jndiFactory =
			(String)env.get( MessageAdapterFactory.INITIAL_CONTEXT );
		jmsFactory =
			(String)env.get( MessageAdapterFactory.CONNECTION_FACTORY );
		url = (String)env.get( MessageAdapterFactory.CONNECTION_URL );
		user = (String)env.get( MessageAdapterFactory.CONNECTION_USER );
		passwd = (String)env.get( MessageAdapterFactory.CONNECTION_PASSWD );
		sendName = (String)env.get( MessageAdapterFactory.DEST_SEND );
		recvName = (String)env.get( MessageAdapterFactory.DEST_RECV );

		connect( jndiFactory, jmsFactory, url, user, passwd );
	}

	/**
	* ＪＭＳに接続します
	* @param	jndiFactory	Jndi名
	* @param	jmsFactory	接続コンテキストファクトリ
	* @param	url			接続URL
	* @param	user		接続ユーザ
	* @param	passwd		接続パスワード
	*/
	private void connect( String jndiFactory, String jmsFactory, String url,
			String user, String passwd )
		throws Exception
	{
		Hashtable<String,String> env = new Hashtable<String,String>();
		env.put( Context.INITIAL_CONTEXT_FACTORY, jndiFactory );
		env.put( Context.PROVIDER_URL, url );

		InitialContext ctx = new InitialContext( env );
        this.jndi = ctx;

		CmnLog.debug( this, "connect jndi:" + jndiFactory + " url:" + url
			+ " connectionFactory:" + jmsFactory
			+ " user=" + user + " passwd:" + passwd
			+ " send:" + sendName + " recv:" + recvName );

		try {
			ConnectionFactory conFactory =
				(ConnectionFactory)ctx.lookup( jmsFactory );
			if ( user != null ) {
				connection = conFactory.createConnection( user, passwd );
			} else {
				connection = conFactory.createConnection();
			}
			connection.setExceptionListener( this );

			sndSession =
				connection.createSession( false, Session.AUTO_ACKNOWLEDGE );
			rcvSession =
				connection.createSession( false, Session.AUTO_ACKNOWLEDGE );
		} catch ( JMSException jex ) {
			Exception lex = jex.getLinkedException();
			if ( lex != null ) {
				CmnLog.error( this, "linked exception:" + lex );
			}
			throw( jex );
		}
	}

	/**
	* 送信リソースを生成します
	*/
	public void createSender() throws Exception {
		createSender( sendName, true );
	}

	/**
	* 送信リソースを生成します
	* @param	queue	送信者
	*/
	public void createSender( String queue ) throws Exception {
		createSender( queue, true );
	}

	/**
	*　送信リソースを生成します
	* @param	queue	送信者
	* @param	use		JNDIの使用有無
	*/
	public abstract void createSender( String queue, boolean use )
		throws Exception;

	/**
	* 受信リソースを生成します
	* @param	cond	受信条件
	* @param	wait	(true: wait false: non wait)
	*/
	public void createReceiver( String cond, boolean wait )
		throws Exception
	{
		createReceiver( recvName, cond, wait, true );
	}

	/**
	* 受信リソースを生成します
	* @param	queue	受信者
	* @param	cond	受信条件
	* @param	wait	(true: wait false: non wait)
	*/
	public void createReceiver( String queue, String cond, boolean wait )
		throws Exception
	{
		createReceiver( queue, cond, wait, true );
	}

	/**
	* 受信リソースを生成します
	* @param	queue	受信者
	* @param	cond	受信条件
	* @param	wait	(true:wait false: non wait)
	* @param	use		JNDIの使用有無
	*/
	public abstract void createReceiver( String queue, String cond, boolean wait,
			boolean use )
		throws Exception;

	/**
	* メッセージングサービスを開始
	*/
	public void start() throws Exception {
		connection.start();
	}

	/**
	* メッセージングサービスを中断します
	*/
	public void stop() throws Exception {
		connection.stop();
	}

	/**
	* メッセージングサービスを閉じます
	*/
	public void close() throws Exception {
		connection.close();
	}

	/**
	* メッセージを送信します
	* @param	msg		送信するメッセージ
	* @param	expiration	メッセージの有効期限
	* @return	waitがtrueの場合は受信したメッセージを、それ以外はヌルを戻す
	*/
	public IMessage writeMessage( IMessage msg, long expiration )
		throws Exception
	{
		JmsMessage jmsg = (JmsMessage )msg;
		if ( wait ) {
			sender.setTimeToLive( expiration );
			sender.send( jmsg.getMessage() );
			return receiveMessage();
		} else {
			sender.send( jmsg.getMessage() );
			return null;
		}
	}

	/**
	* メッセージを受信します
	* @return	受信したメッセージ
	*/
	public IMessage receiveMessage() throws Exception {
		Message msg = receiver.receive( recv_timeout );
		JmsMessage ret = new JmsMessage( msg );
		return ret;
	}

	/**
	* メッセージの受信コールバック
	* @param	message	受信したメッセージ
	*/
	public void onMessage( Message message ) {
		try {
			JmsMessage msg = new JmsMessage( message );
			Iterator iterator = listeners.iterator();
			IReceiveListener listener;
            while ( iterator.hasNext() ) {
                listener = (IReceiveListener )iterator.next();
                listener.onMessage( this, msg );
            }
		} catch ( Exception ex ) {
			CmnLog.error( this, "onMessage", ex );
		}
	}

	/**
	* JMSException コールバック
	* @param	jmse	Exception情報
	*/
	public void onException( JMSException jmse ) {
		for ( int i = 0; i < exceptions.size(); i++ ) {
			IExceptionListener l = (IExceptionListener )exceptions.get( i );
			l.onException( jmse );
		}
	}

	/**
	* メッセージインスタンスを生成します
	* @param	type	メッセージタイプ
	* @return	メッセージインスタンス
	*/
	public IMessage createMessage( IMessage.Type type ) throws Exception {
		JmsMessage ret = new JmsMessage( type, sndSession );
		return ret;
	}

	/**
	* メッセージの複製を作成します
	* @param	msg		コピー元
	* @return	複製されたメッセージ
	*/
	public IMessage copyMessage( IMessage msg ) throws Exception {
		JmsMessage ret = new JmsMessage( msg.getType(), sndSession );
		ret.copyMessage( msg );
		return ret;
	}

	/**
	* メッセージの受信コールバックを設定します。
	* @param	listener	メッセージの受信コールバック
	*/
	public void addReceiveListener( IReceiveListener listener ) {
		listeners.add( listener );
	}

	/**
	* 例外の受信コールバックを登録します。
	* @param	listener	例外の受信コールバック
	*/
	public void addExceptionListener( IExceptionListener listener ) {
		exceptions.add( listener );
	}

	/**
	* 受信タイムアウトを設定します。
	* @param	millisec	受信タイムアウト(ミリ秒)
	*/
	public void setReceiveTimeout( int millisec ) {
		recv_timeout = millisec;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param    config  設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		config.addChild( MessageAdapterFactory.INITIAL_CONTEXT, jndiFactory );
		config.addChild( MessageAdapterFactory.CONNECTION_FACTORY, jmsFactory );
		config.addChild( MessageAdapterFactory.CONNECTION_URL, url );
		config.addChild( MessageAdapterFactory.CONNECTION_USER, user );
		config.addChild( MessageAdapterFactory.CONNECTION_PASSWD, passwd );
		if ( sendName != null ) {
			config.addChild( MessageAdapterFactory.DEST_SEND, sendName );
		}
		if ( recvName != null ) {
			config.addChild( MessageAdapterFactory.DEST_RECV, recvName );
		}
	}

}
